@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 
-# $Id: clamdsh.pl,v 1.5 2007/05/09 02:21:07 okamura Exp $
+# $Id: clamdsh.pl,v 1.7 2007/05/18 18:45:11 okamura Exp $
 
 require 5.8.0;
 use strict;
@@ -19,7 +19,6 @@ use Time::HiRes;
 use POSIX	qw(strftime);
 use Sys::Hostname;
 use File::Basename;
-#use Time::Local;
 
 # プロジェクト
 
@@ -38,7 +37,7 @@ use vars qw(
 # 定数
 #-------------------------------------------------------------------------------
 # バージョン
-$Version = 'clamdsh.pl v1.0.1';
+$Version = 'clamdsh.pl v1.0.2';
 
 # 終了ステータス
 %ExitStatus = (
@@ -468,6 +467,32 @@ sub InterpretString {
 			$status = 2;	# シングルクォート内へ
 		}
 	}
+	# 不完全な終わり方をしている場合
+	unless ($status == 0) {
+		# ダブルクォートが開きっぱなしで終わっている場合
+		if ($status == 1 or $status == 6 or $status == 7) {
+			ClamAV::Shell::UI::Printf("Double quotation is not closed.\n");
+		}
+		# シングルクォートが開きっぱなしで終わっている場合
+		if ($status == 2 or $status == 8) {
+			ClamAV::Shell::UI::Printf("Single quotation is not closed.\n");
+		}
+		# エスケープされぱなしで終わっている場合
+		if ($status == 3 or $status == 6 or $status == 8) {
+			# 警告せずにエスケープを加える。
+			push @interpreted, '\\';
+		}
+		# 変数が指示されぱなしで終わっている場合
+		if ($status == 4 or $status == 7) {
+			# 警告せずに $ を加える。
+			push @interpreted, '$';
+		}
+		# 引き数セパレータ内で終わっている場合
+		if ($status == 5) {
+			# なにもしない
+			1;
+		}
+	}
 	push @args, join('', @interpreted);
 
 	# 先頭の空白要素は省略する
@@ -656,7 +681,9 @@ sub _SendAndRecv {
 	my	($clamdCmd, @args) = @_;
 
 	unless (scalar(@args) == 0) {
-		ClamAV::Shell::UI::Printf("Usage: %s", $clamdCmd);
+		ClamAV::Shell::UI::Printf(
+			"[USAGE]\n%s", $CmdInfo{$clamdCmd}->{'usage'}
+		);
 		return 1;
 	}
 
@@ -755,9 +782,7 @@ sub _SCAN {
 
 	unless (scalar(@args) == 1) {
 		ClamAV::Shell::UI::Printf(
-			"Usage: %s PATH
-    PATH - The path of a file or directory.",
-			$clamdCmd
+			"[USAGE]\n%s", $CmdInfo{$clamdCmd}->{'usage'}
 		);
 		return 1;
 	}
@@ -865,8 +890,7 @@ sub STREAM {
 
 	unless (scalar(@args) == 1) {
 		ClamAV::Shell::UI::Printf(
-			"Usage: STREAM PATH
-    PATH - The path of a file or directory."
+			"[USAGE]\n%s", $CmdInfo{'STREAM'}->{'usage'}
 		);
 		return 1;
 	}
@@ -993,8 +1017,7 @@ sub SESSION {
 
 	unless (scalar(@args) == 1) {
 		ClamAV::Shell::UI::Printf(
-			"Usage: SESSION PATH
-    PATH - The path of a file which is writen clamd-commands."
+			"[USAGE]\n%s", $CmdInfo{'SESSION'}->{'usage'}
 		);
 		return 1;
 	}
@@ -1315,7 +1338,7 @@ $HistoryNumber = 0;
 				and
 			defined($ClamAV::Shell::Option{'TCPAddr'})
 		) {
-			sprintf("%s:%s:",
+			sprintf("%s:%s",
 				$ClamAV::Shell::Option{'TCPAddr'},
 				$ClamAV::Shell::Option{'TCPSocket'}
 			)
@@ -1576,7 +1599,9 @@ if ($RunAsCommand) {
 		'bufsiz=i'	=> \&ClamAV::Shell::Option::SetOption4CmdLine,
 		'recvTimeout=i'	=> \&ClamAV::Shell::Option::SetOption4CmdLine,
 		#--- 情報表示 ----
-		'help|h'	=> sub { pod2usage(-verbose => 2) },
+		'help|h'	=> sub {
+			pod2usage(-verbose => 2, exit => $ExitStatus{NoError})
+		},
 		'version'	=> sub {
 			print <<EOF;
 $Version
@@ -1586,7 +1611,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 EOF
 			exit $ExitStatus{NoError};
 		},
-	) or pod2usage($ExitStatus{UknownOpts});
+	) or pod2usage(-verbose => 0, exit => $ExitStatus{UknownOpts});
 }
 
 #-------------------------------------------------------------------------------
@@ -1742,6 +1767,20 @@ DEFAULT: 20
 
 =back
 
+=head1 COMMAND ISSUE
+
+A command line of B<clamdsh.pl> is a sequence by blank-separated words.
+The first word specifies the command to issue.
+The remaining words are aruguments to the invoked command.
+Each command lines terminate with line feed.
+
+It is just like a OS shell.
+But B<clamdsh.pl> doesn't support multi lines command.
+Backslash on end of line doesn't escape line feed,
+double or single quote can't include line feeds.
+
+You can display the list of commands by `help' command.
+
 =head1 QUOTING
 
 Single quotes and double quotes are used to remove the special meaning of
@@ -1859,7 +1898,15 @@ AND size of the file is greater than the value of C<StreamMaxLength> in clamd.
 
 =back
 
-This problem cannot be evaded. Use STREAM command carefully.
+This problem can't be evaded. Use STREAM command carefully.
+
+=head1 EXIT STATUS
+
+0 - No error.
+
+1 - Unknown option is given.
+
+2 - Bad option value is given.
 
 =head1 SEE ALSO
 
